/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 *    Author Quan Sun (quan.sun.nz@gmail.com)
 *    Copyright (C) 2013 University of Waikato, Hamilton, New Zealand
 *
 */

package artforests.core;
import java.util.Random;
import weka.core.Instances;
import weka.filters.Filter;

public class ARTForests extends AbstractRanker {
    private int m_T = 10;
    private Ranker[] m_WeakRankers;
    private int m_RandomSeed = 1;
    private int m_NumMinInstances = 15;
    private boolean m_UseMedian = false;
    private double m_BaggingPercentage = 100.0;
    private int m_K = 0;

    public void setK(int k) {
        m_K = k;
    }    
    
    public void setBaggingPercentage(double p) {
        if (m_BaggingPercentage > 100 || m_BaggingPercentage < 5) {
            m_BaggingPercentage = 100;
        } else {
            m_BaggingPercentage = p;
        }
    }

    public void setUseMedian(boolean m) {
        m_UseMedian = m;
    }

    public void setNumMinInstances(int m) {
        m_NumMinInstances = m;
    }

    @Override
    public void buildRanker(Instances metaData) throws Exception {
        Random r = new Random(m_RandomSeed);
        Instances workingData = new Instances(metaData);
        m_WeakRankers = new Ranker[m_T];
        for (int i = 0; i < m_T; i++) {
            Instances baggingSample = workingData.resampleWithWeights(r);
            if (m_BaggingPercentage < 100.0) {
                weka.filters.unsupervised.instance.Resample res = new weka.filters.unsupervised.instance.Resample();
                res.setSampleSizePercent(m_BaggingPercentage);
                res.setNoReplacement(false);
                res.setInputFormat(baggingSample);
                baggingSample = Filter.useFilter(baggingSample, res);
            }
            ART ranker = new ART();           
            ranker.setMiniLeaf(m_NumMinInstances);
            ranker.setK(m_K);
            ranker.setRandomSeed(i);
            ranker.setUseMedian(m_UseMedian);
            m_WeakRankers[i] = ranker;
            m_WeakRankers[i].buildRanker(baggingSample);
        }
    }

    public void setNumIterations(int t) {
        m_T = t;
    }

    @Override
    public double[] recommendRanking(MultiTargetInstance metaInst) throws Exception {
        double[] pred = m_WeakRankers[0].recommendRanking(metaInst);
        if (m_T > 1) {
            for (int i = 1; i < m_T; i++) {
                double[] pred_i = m_WeakRankers[i].recommendRanking(metaInst);
                for (int j = 0; j < pred.length; j++) {
                    pred[j] += pred_i[j];
                }
            }
        }
        for (int j = 0; j < pred.length; j++) {
            pred[j] /= (1.0 * m_T);
        }
        return Tools.doubleArrayToRanking(pred, EvalTool.RANKING_ALGO);
    }
}
