/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 *    Author Quan Sun (quan.sun.nz@gmail.com)
 *    Copyright (C) 2013 University of Waikato, Hamilton, New Zealand
 *
 */
package artforests.core;

import java.io.*;
import java.util.*;
import jsc.util.Rank;
import weka.core.*;
import weka.core.converters.ArffLoader;
import weka.core.converters.CSVLoader;

public class Tools {

    public static ArrayList getRanks(ArrayList<Double> values) {
        ArrayList<Double> sortedValues = new ArrayList(values);
        Collections.sort(sortedValues);
        ArrayList<Double> ranks = new ArrayList();
        for (int i = 0; i < values.size(); i++) {
            double r = sortedValues.indexOf(values.get(i)) + 1;
            ranks.add(r);
        }
        return ranks;
    }

    public static Instances loadCSVInstances(String arffPath) throws Exception {
        CSVLoader loader = new CSVLoader();
        loader.setSource(new File(arffPath));
        Instances data = loader.getDataSet();
        data.setClassIndex(data.numAttributes() - 1);
        return data;
    }

    public static Instances loadARFFInstances(String arffPath) throws Exception {
        ArffLoader loader = new ArffLoader();
        loader.setSource(new File(arffPath));
        Instances data = loader.getDataSet();
        data.setClassIndex(data.numAttributes() - 1);
        return data;
    }

    private static class TempValue implements Comparable {

        double Value = Double.NaN;
        double Rank = -1;
        int Index = -1;
        private boolean m_SortByValue = true;

        public TempValue(int index, double s, double r) {
            Index = index;
            Value = s;
            Rank = r;
        }

        public void setSortByScore(boolean b) {
            m_SortByValue = b;
        }

        @Override
        public int compareTo(Object o) {
            if (m_SortByValue) {
                TempValue other = (TempValue) o;
                if (other.Value > Value) {
                    return -1;
                } else if (other.Value == Value) {
                    return 0;
                } else {
                    return 1;
                }
            } else {
                TempValue other = (TempValue) o;
                if (other.Index > Index) {
                    return -1;
                } else if (other.Index == Index) {
                    return 0;
                } else {
                    return 1;
                }
            }
        }
    }

    public static double[] doubleArrayToRanking(double[] values, int rankingAlgo) {

        if (rankingAlgo == 2) {

            double[] ranks = null;
            Rank rank = null;
            try {
                rank = new Rank(values, 0.0001);
                //rank = new Rank(values, 0); 
                //rank = new Rank(values.length, values, 0); 
                ranks = rank.getRanks();
            } catch (Exception ee) {
                for (int xxx = 0; xxx < values.length; xxx++) {
                    System.out.print(values[xxx] + " ");
                }
                System.out.println();
                System.err.println(ee.toString());
            }
            return ranks;

        } else {

            TempValue[] ranks = new TempValue[values.length];
            for (int i = 0; i < values.length; i++) {
                ranks[i] = new TempValue(i, values[i], -1);
            }
            Arrays.sort(ranks);
            for (int i = 0; i < values.length; i++) {
                ranks[i].Rank = (i + 1);
                ranks[i].m_SortByValue = false;
            }
            Arrays.sort(ranks);
            double[] ranking = new double[values.length];
            for (int i = 0; i < values.length; i++) {
                //ranking[i] = ranks[i].Rank; //old
                ranking[i] = values.length - ranks[i].Rank + 1; // 16/01/2013
            }
            return ranking;
        }
    }

    public static double[] doubleArrayToRanking(double[] values) {
        Rank rank = new Rank(values, 0.0001);
        return rank.getRanks();
    }

    public static ArrayList<String> readCSVFile(String filePath) {
        ArrayList<String> lines = new ArrayList<String>();
        File file = new File(filePath);
        BufferedReader reader = null;
        try {
            reader = new BufferedReader(new FileReader(file));
            String text = null;
            // repeat until all lines is read
            while ((text = reader.readLine()) != null) {
                String val = text.trim();
                lines.add(val);
            }
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            try {
                if (reader != null) {
                    reader.close();
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return lines;
    }


    public static void saveStringBuilder(StringBuilder sb, String filePath) throws Exception {
        BufferedWriter writer = new BufferedWriter(new FileWriter(filePath));
        writer.write(sb.toString());
        writer.flush();
        writer.close();
    }

    public static String newLine() {
        return System.getProperty("line.separator");
    }
}
